/**
 * @file
 * Provides JavaScript for Entity Form Monitor.
 */

(function ($, Drupal, drupalSettings) {

  /**
   * Test
   *
   * @namespace
   */
  Drupal.entityFormMonitor = Drupal.entityFormMonitor || {

    /**
     * The update monitor.
     *
     * This will be set in Drupal.entityFormMonitor.startMonitoring().
     */
    timer: null,

    /**
     * Start monitoring the entity forms.
     *
     * @name Drupal.entityFormMonitor.startMonitoring
     */
    startMonitoring: function(interval) {
      // Start the update poller. We only need to do this once even if a
      // later AJAX request adds another entity form, because the
      // Drupal.entityFormMonitor.checkForms() method looks globally
      // at all current forms on the page.
      if (this.timer === null) {
        console.log("Setting monitoring interval to " + (interval / 1000) + " seconds");
        this.timer = setInterval(this.checkForm, interval);
      }
    },

    /**
     * Stop monitoring the entity forms.
     *
     * @name Drupal.entityFormMonitor.stopMonitoring
     */
    stopMonitoring: function() {
      console.log("Stopping monitoring.");
      clearInterval(this.timer);
    },

    /**
     * Mark an entity form as changed.
     *
     * @name Drupal.entityFormMonitor.markFormChanged
     *
     * @param {jQuery.Event} event
     *   The event triggered, most likely a `change` event.
     */
    markFormChanged: function (event) {
      console.log("Detected a changed field value in an entity form.");
      $entityForm = $(event.target).parents('[data-entity-form-monitor]').first();
      $entityForm.data('entity-form-changed', true);
    },

    /**
     * Checks if any entity forms have been changed.
     *
     * @name Drupal.entityFormMonitor.isFormChanged
     */
    isFormChanged: function () {
      return $('[data-entity-form-monitor]:data(entity-form-changed)').length > 0;
    },

    /**
     * Invalidates the current form and either reloads the page or shows a modal.
     *
     * @name Drupal.entityFormMonitor.invalidateForm
     */
    invalidateForm: function () {
      // Disable all submit buttons in the entity forms to prevent form submission.
      $('[data-entity-form-monitor] input[type="submit"]').prop('disabled', true);
      // Hide the actions in the entity forms.
      //$('[data-entity-form-monitor] .form-actions').hide();

      // @todo Convert this dialog and message to something that is injected via AJAX commands.
      if (this.isFormChanged()) {
        console.log('Detected changes made in the forms, showing dialog.');
        var $dialog = $('<div id="entity-form-update-changed-dialog" title="Form out of date">This form is out of date and cannot be submitted. You must reload the form. Any changes you may have saved will be lost when the page reloads.</div>');
        $('body').append($dialog);
        $dialog.dialog({
          resizable: false,
          height: "auto",
          width: 400,
          modal: true,
          buttons: {
            "Reload form": function () {
              console.log('User requested to reload the form.');
              location.reload();
            },
            Cancel: function () {
              // Display a warning message on the form.
              var $warning = $('<div class="messages messages--error">This form is out of date and cannot be submitted. You must reload the form. Any changes you may have saved will be lost when the page reloads.</div>');
              $('form').first().before($warning);
              // Close and remove the dialog.
              $(this).dialog("close");
              $dialog.remove();
            }
          }
        });
      }
      else {
        console.log('Did not detect changes made in the forms, reloading the page automatically.');
        location.reload();
      }
    },

    /**
     * Monitor updates to all entity forms on the current page.
     *
     * @name Drupal.entityFormMonitor.monitorForms
     */
    checkForm: function () {
      var $entityForms = $('[data-entity-form-monitor]');
      if ($entityForms.length) {
        // Gather all the entity IDs and last changed timestamps.
        var entityData = [];
        $entityForms.each(function() {
          var entityId = $(this).data('entity-form-monitor');
          entityData[entityId] = $(this).data('entity-last-changed');
        }).get();

        console.log((new Date().getTime() / 1000) + ': polling for updates for ' + Object.keys(entityData).join(' '));

        // Send the AJAX request to get the current entity updated timestamps.
        $.ajax({
          url: drupalSettings.path.baseUrl + 'entity-form-monitor',
          data: {
            'entities': Object.keys(entityData)
          },
          type: 'POST',
          error: function () {
            // If we encountered an error, stop monitoring.
            this.stopMonitoring();
          },
          success: function (updatedData) {
            $.each(updatedData, function(entityId, lastChanged) {
              if (lastChanged === false || lastChanged > entityData[entityId]) {
                // A false value means the entity was deleted.
                Drupal.entityFormMonitor.stopMonitoring();
                Drupal.entityFormMonitor.invalidateForm();
                return false;
              }
            });
          }
        });
      }
    }

  };

  /**
   * Attach behaviors to monitor entity forms.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches triggers.
   * @prop {Drupal~behaviorDetach} detach
   *   Detaches triggers.
   */
  Drupal.behaviors.entityFormMonitor = {
    attach: function (context, settings) {
      var $entityForms = $(context).find('[data-entity-form-monitor]');
      if ($entityForms.length) {
        // Add a change handler that will help us determine if any inputs
        // inside the entity forms have changed values.
        $entityForms.find(':input').once().on('change input paste', Drupal.entityFormMonitor.markFormChanged);
        // Start the update monitor. We only need to do this once even if a
        // later AJAX request adds another entity form, because the
        // Drupal.entityFormUpdateCheck.checkForms() method looks globally
        // at all current forms on the page.
        var interval = settings.entityFormMonitor.interval * 1000;
        Drupal.entityFormMonitor.startMonitoring(interval);
      }
    },
    detach: function (context) {
      //var $entityForms = $(context).find('[data-entity-form-update-check-entity]');
      //$entityForms.find(':input').off('change', Drupal.entityFormMonitor.markFormChanged);
    }
  };

})(jQuery, Drupal, drupalSettings);
