<?php

namespace Drupal\entity_form_monitor\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Defines a controller to find the latest update values of entities.
 */
class EntityMonitor extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Constructs an EntityMonitor object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Returns the entity updates.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JsonResponse object.
   */
  public function getUpdates(Request $request) {
    $post = $request->request->all();

    if (empty($post['entities']) || !is_array($post['entities'])) {
      throw new AccessDeniedHttpException();
    }

    $updates = [];
    foreach ($post['entities'] as $entity_id) {
      list($entity_type, $id) = explode(':', $entity_id);
      // @todo Should we be using loadUnchanged() here to ensure we have the absolute latest data?
      if ($entity = $this->entityTypeManager()->getStorage($entity_type)->load($id)) {
        // Double check that the user has access to edit the entity.
        if (!$entity->access('edit')) {
          throw new AccessDeniedHttpException();
        }
        $updates[$entity_id] = $entity->getChangedTime();
      }
      else {
        // Entity was deleted.
        $updates[$entity_id] = FALSE;
      }
    }

    return new JsonResponse($updates);
  }

}
